/*
  CommentEditorPopup.h

  This file is part of Charm, a task-based time tracking application.

  Copyright (C) 2015-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com

  Author: Mathias Hasselmann <mathias.hasselmann@kdab.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMMENTEDITORPOPUP_H
#define COMMENTEDITORPOPUP_H

#include <QDialog>

#include "Core/Event.h"

namespace Ui {
class CommentEditorPopup;
}

class CommentEditorPopup : public QDialog
{
    Q_OBJECT

public:
    explicit CommentEditorPopup( QWidget *parent = nullptr );
    ~CommentEditorPopup();

public Q_SLOTS:
    void loadEvent( EventId id );
    void accept() override;

private:
    Ui::CommentEditorPopup *ui;
    EventId m_id;
};

#endif // COMMENTEDITORPOPUP_H
